#define VERSION "0.9"

/*
Better X font selector using Qt library

Author: Jacek Pliszka <pliszka@fuw.edu.pl>, 11 Oct 2000.
Patches: Enrique Perez-Terron <enrio@online.no>, v0.9, 30 Nov 2001.


Licence: GPL
 */
#include "xfontselector.h"

#include <qapplication.h>

#include <qlabel.h>
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qtextview.h>
#include <qlayout.h>
#include <qcombobox.h>

#include <X11/Xlib.h>

#include <iostream.h>
#include <stdlib.h>

// This is help text

static const char* helptext[] ={
  "<b>Short Instruction</b><br>"
  "Start with resizing the main program window for Your convenience."
  " Then select whether font should be scalable or not,"
  " select acceptable font sizes, widths, slants, heights and names."
  " Then click <b>Show</b>. "
  " A list of fonts matching the criteria should appear on the right "
  " together with some test text (may not fit fully in the window)."
  " Then select the fonts You like the most and click <b>Print</b>."
  " The final list of fonts will appear in the terminal in which"
  " program was started."
  " The header will be <b>Selected fonts</b>."
  " Cut and paste those font names to .Xdefaults and other places"
  " according to Your needs.<br>"
  " At the moment only iso8859-2 is working(lack of time)."
  " Any contribution is welcome."
  "<br><br>Version " VERSION ", Jacek Pliszka pliszka@fuw.edu.pl, 11 Oct 2000<br>"
  "Web Page: http://www.fuw.edu.pl/~pliszka/xfontselector"  ,
  "In preparation"
};

// Structure containing information about encodings
struct Encoding{
  int code;
  char *fontpattern;
  char *codecname;
  char *testtext;
  QFont::CharSet charset; 
};

int myencoding=2;
int currentEncoding=-1;		// Avoid querying the X server repeatedly about the same encoding

const Encoding knownEncodings[]={
  {0,"*",       "ISO 8859-1","The Quick Brown Fox Jumps Over The Lazy Dog",QFont::AnyCharSet},
  {1,"*8859-1", "ISO 8859-1","The Quick Brown Fox Jumps Over The Lazy Dog",QFont::ISO_8859_1},
  {2,"*8859-2", "ISO 8859-2","cia si ja, si",QFont::ISO_8859_2},
  {3,"*10646-*", "ISO 10646","Math symbols:\342\210\200\342\210\201\342\210\202\342\210\203\342\210\204",QFont::Unicode},
  {-1,NULL,NULL,NULL}
};

// This function defines how fonts should be sorted

int FontInfoList::compareItems ( FontInfoList::Item item1, FontInfoList::Item item2 ){
  FontInfo *f1=(FontInfo *) item1;
  FontInfo *f2=(FontInfo *) item2;

  // first size
  int res=f1->size.compare(f2->size);

  // slant
  if(!res) res=f1->slant.compare(f2->slant);

  // weight
  if(!res) res=f1->weight.compare(f2->weight);

  // width
  if(!res) res=f1->width.compare(f2->width);

  // name
  if(!res) res=f1->name.compare(f2->name);

  return(res);
}

// this routine queries the X server for font list and parses it
void FontSelector::get_font_list(int encoding){
  Display *display;
  char **lista;
  int fonts_count;

 // Avoid repetitions
  if (encoding == currentEncoding)
    return;
  if (currentEncoding != -1) {
    fonts->clear();		// has autoDelete set to true
    names->clear();
    weights->clear();
    slants->clear();
    widths->clear();
    namelist->clear();
    weightlist->clear();
    slantlist->clear();
    widthlist->clear();
  }

  // read fonts from Xserver
  display=XOpenDisplay(NULL);
  char *pattern=(knownEncodings[encoding]).fontpattern;

  cout << "Querying  font server for " << pattern<< "fonts...\n";

  lista=XListFonts(display,pattern, 10000,&fonts_count);
  // No, we won't handle 10000 fonts!!
  if(fonts_count==10000){
    cerr << "Font limit reached!!! I cannot handle more than 10k fonts!!\n";
    exit(0);
  }
  for(int i=0;i<fonts_count;i++){
    FontInfo finfo;

    finfo.font=QString(lista[i]);

    //    cout << "Font:" << finfo.font << endl;

    // Now parse font names, there must a better method .... sighhh...

    int lindex=finfo.font.find("-")+1;
    if(lindex<1) continue;
    //    cout << lindex <<" ";
    int mindex=finfo.font.find("-",lindex)+1;
    if(mindex<1) continue;
    //    cout << rindex <<" ";
    int rindex=finfo.font.find("-",mindex);
    if(rindex<0) continue;
    // cout << rindex <<endl;
    finfo.family=finfo.font.mid(mindex,rindex-mindex);
    finfo.name=finfo.font.mid(lindex,rindex-lindex);
    // cout << "Name: " << finfo.name << endl;

#define string_to_next_minus(where) \
    lindex=rindex+1; \
    rindex=finfo.font.find("-",lindex); \
    where=finfo.font.mid(lindex,rindex-lindex);

    string_to_next_minus(finfo.weight);

    // cout << "Weight: " << finfo.weight << endl;

    string_to_next_minus(finfo.slant);

    //cout << "Slant: " <<finfo.slant << endl;

    string_to_next_minus(finfo.width); 

    //cout << "Width: " << finfo.width << endl;

    // skip one minus
    lindex=rindex+1;
    rindex=finfo.font.find("-",lindex);

    string_to_next_minus(finfo.size);

    finfo.sizestart=lindex;
    finfo.sizeend=rindex;

    //cout << "Size: " << finfo.size << endl;

    // If names are OK copy them to my structures
    if(finfo.size.length()>0){
      FontInfo *pfinfo=new FontInfo();
      *pfinfo=finfo;
      cout << "Found: " << finfo.font << "\n";
      fonts->append(pfinfo);
      // check whether it is a new name/weight/sland/width

#define add_if_unknown(field,list) \
      if(! list->contains(pfinfo->field))  \
	list->append(pfinfo->field);
	
	add_if_unknown(name,names);
	add_if_unknown(weight,weights);
	add_if_unknown(slant,slants);
	add_if_unknown(width,widths);

    }
  }
  // Sort fonts for conveneince
  fonts->sort();
  names->sort();
  weights->sort();
  slants->sort();
  widths->sort();

  if (currentEncoding > -1) {
    namelist->insertStringList(*names);
    weightlist->insertStringList(*weights);
    slantlist->insertStringList(*slants);
    widthlist->insertStringList(*widths);
  }

  // We are done with X server
  XFreeFontNames(lista);
  XCloseDisplay(display);

  currentEncoding = encoding;
}

// This makes encoding combo
void insert_encodings(QComboBox *combo){
  int i,n;
  for ( i=0; (n=knownEncodings[i].code)>-1; i++ ) {
    // Insert given thing at position i
    combo->insertItem(knownEncodings[i].fontpattern,i);
  }
  combo->setCurrentItem(myencoding);
}

// The main, huge, ugly routine
FontSelector::FontSelector()
    : QWidget( 0, 0 )
{
  fonts = new FontInfoList;
  names = new QStringList;
  weights = new QStringList;
  slants = new QStringList;
  widths = new QStringList;

  fonts->setAutoDelete(true);	// So we can clear() *fonts when a new encoding is set.

  // get fonts from X server
  currentEncoding = -1;
  get_font_list(myencoding);

  QLabel *mylabel;
  QPushButton *mybutton;
  QCheckBox *mycheckbox;

  // Now the screen

  // main splitter
  mainsplitter = new QSplitter( QSplitter::Horizontal, this );

  // First the left part
  verticalsplitter1 = new QSplitter( QSplitter::Vertical, mainsplitter );

  //  Due to SLOT thing this has to be macro not a function
#define add_my_button(name,parent,action,minx,miny,maxx,maxy) \
  mybutton=new QPushButton( name, parent ); \
  connect(mybutton,SIGNAL(clicked()),SLOT(action)); \
  mybutton->setMinimumSize( minx, miny ); \
  mybutton->setMaximumSize( maxx, maxy ); 

  // Now it is easy to add 4 buttons ...
  add_my_button("Quit", verticalsplitter1,quit(), 30, 10,60, 20 );
  
  add_my_button("Show", verticalsplitter1,showFonts(),30,10,60,20);

  add_my_button("Print",verticalsplitter1,printFonts(),30,10,60,20);

  add_my_button("Help", verticalsplitter1,help(), 30,10,60,20);

  // OK, ugly part - help window
  helpPopup=new QFrame( this ,0, WType_Popup);
  helpPopup->setFrameStyle( QFrame::PopupPanel|QFrame::Raised );
  helpPopup->resize(400,250);

  QVBoxLayout * helplayout=new QVBoxLayout(helpPopup);

  QTextView * textv=new QTextView(helpPopup);
  helplayout->addWidget(textv);
  textv->setText(helptext[0]);
  textv->setMinimumSize(350,200);

  add_my_button("Close", helpPopup,close(),50,30,200,30);
  helplayout->addWidget(mybutton);

  // OK, let's generate sizes list, is there any need for >36 ??? <5???
  sizes = new QStringList();
  QString tmpString;

  for(int i=5;i<37;i++){
    sizes -> append(tmpString.setNum(i));
  }
  
  // OK, again macro since this repeats 4 times
#define add_my_list(whichlist,parent,strlist,minx,miny,maxx,maxy) \
  whichlist = new QListBox( parent ); \
  whichlist->setFocusPolicy( QWidget::StrongFocus ); \
  whichlist->setMultiSelection(true); \
  whichlist->setColumnMode(1);     \
  whichlist->insertStringList(*strlist); \
  whichlist->setMaximumSize( maxx, maxy); \
  whichlist->setMinimumSize( minx, miny ); 

  // size list
  add_my_list(sizelist,verticalsplitter1,sizes,25,100,100,2000);
  
  // Now middle part
  verticalsplitter2 = new QSplitter( QSplitter::Vertical, mainsplitter );

  // OK ,encoding combo, not working :)
  QComboBox *mycombo= new QComboBox(false,verticalsplitter2,"Encoding");
  insert_encodings(mycombo);
  mycombo->setMaximumSize( 160, 25 );
  mycombo->setMinimumSize( 60, 20 );
  connect(mycombo, SIGNAL(activated(int)), this, SLOT(get_font_list(int)));

  // OK, now chose whether fonts should be scalable or not
  scalablefonts=new QCheckBox( "Scalable", verticalsplitter2);  
  scalablefonts->setMaximumSize( 160, 20 );
  scalablefonts->setMinimumSize( 60, 12 );

  // Now 4 lists

  add_my_list(weightlist,verticalsplitter2,weights,60,30,160,120);

  add_my_list(slantlist,verticalsplitter2,slants,60,30,160,120);

  add_my_list(widthlist,verticalsplitter2,widths,60,30,160,120);

  add_my_list(namelist,verticalsplitter2,names,60,100,160,2000);

  // OK, this is the most right box with font list/test text
  matchingfonts=new QListView(mainsplitter); 
  matchingfonts->setMultiSelection(true);
  matchingfonts->addColumn( "Font" );
  matchingfonts->addColumn( "Text" );
  matchingfonts->setAllColumnsShowFocus( TRUE );
}

// OK, this is to force the mainsplitter to resize

void FontSelector::resizeSelector(){
   mainsplitter->resize(this->size()); 
}

void FontSelector::resizeEvent( QResizeEvent * )
{
  resizeSelector();
}

// This makes the list of selected elements

void copySelected(QListBox *list,QStringList *selected ){
  int cnt;
  cnt=list->count();
  for(cnt=0;cnt<list->count();cnt++){
    if(list->isSelected(cnt)){
      selected->append(list->text(cnt));
      //      cout << "Selected: " << list->text(cnt) << endl;
    }
  }
}

// This this generates the list of fonts matching criteria
void FontSelector::write_out_matching_fonts(QString fontname,
					    QStringList* printed){
  if(!(printed->contains(fontname))){
    printed->append(fontname);
    cout << fontname << endl;
    (void)new FontListItem(matchingfonts,fontname);
  }
}

// OK, this is again ugly part, it generates list of matching fonts
// and shows it

void FontSelector::showFonts(void)
{
  // This should be nice since copySelected should allocate as well
#define generateSelectedList(origlist,selectedlist) \
  QStringList selectedlist; \
  copySelected(origlist,&selectedlist);

  generateSelectedList(sizelist,sizesSelected);
  generateSelectedList(widthlist,widthsSelected);
  generateSelectedList(weightlist,weightsSelected);
  generateSelectedList(slantlist,slantsSelected);
  generateSelectedList(namelist,namesSelected);

  QStringList fontsAlreadyPrinted;

  FontInfo *curr;
  cout << endl << "All fonts matching criteria:" <<endl <<endl;

  matchingfonts->clear();

  // OK, now loop over fonts
  curr=fonts->first();
  while(curr !=NULL){
    // Check if the font matches our criteria
    if(namesSelected.contains(curr->name) &&
       widthsSelected.contains(curr->width) &&
       weightsSelected.contains(curr->weight) &&
       slantsSelected.contains(curr->slant) 
       ){
      // It does, now chek if it is scalable or not
      if(scalablefonts->isChecked()){
	if(curr->size=="0"){  // scalable are those with "0" size
	  // Loop over selected sizes
	  QValueListIterator<QString> currsize = sizesSelected.begin();
	  for ( ; currsize != sizesSelected.end(); ++currsize ) {
	    int intWeight=QFont::Normal;
#define weight_translation(name,code)\
	    if((curr->weight)==name) intWeight=QFont::code;
	    
	    weight_translation("light",Light);
	    weight_translation("demibold",DemiBold);
	    weight_translation("bold",Bold);
	    weight_translation("black",Black);	  
	    
	    //	    if((curr->weight)=="light") intWeight=QFont::Light;
	    //if((curr->weight)=="demibold") intWeight=QFont::DemiBold;
	    //if((curr->weight)=="bold") intWeight=QFont::Bold;
	    //if((curr->weight)=="black") intWeight=QFont::Black;
	    
	    bool boolItalic=false;
	    if((curr->slant)=="o") boolItalic=true;
	    if((curr->slant)=="i") boolItalic=true;
	    
	    // Set the things and find the match
	    QFont font=QFont(curr->family, (*currsize).toInt(),
			     intWeight,boolItalic);
	    // Force the proper charset
	    font.setCharSet( knownEncodings[myencoding].charset);
	    // and find out the raw X name
	    QString currfname(font.rawName());
	    
	    // and write out the result
	    write_out_matching_fonts(currfname,&fontsAlreadyPrinted);
	  };
	};
      } else if(sizesSelected.contains(curr->size)){
	// If it is not scalable then the size should match
	write_out_matching_fonts(curr->font,&fontsAlreadyPrinted);
      }
    }
    curr=fonts->next();  
  }
}

// This routine prints out the fonts selected with mouse from matching ones
void FontSelector::printFonts(void)
{
  cout << endl << "Selected fonts: " << endl<<endl;

  QListViewItem * curr = matchingfonts->firstChild();
  for ( ; curr != NULL; curr=curr->nextSibling() ) {
    FontListItem *currf = (FontListItem*) curr;
    if(currf->isSelected()){
      cout << currf->font() << endl;
    }
  }
}

// Well, this is help
void FontSelector::help(void)
{
  helpPopup->show();
}

// And this is quit
void FontSelector::quit(void)
{
  exit(1);
}

//**********************************************************************

// OK, this creates FontListItem which is font name and test text

FontListItem::FontListItem( QListView *parent, QString f )
    : QListViewItem( parent )
{
  QString testtext;
  if (knownEncodings[currentEncoding].charset == QFont::Unicode)
    testtext = QString::fromUtf8(knownEncodings[currentEncoding].testtext);
  else
    testtext = (QTextCodec::codecForName(knownEncodings[currentEncoding].codecname))-> toUnicode(knownEncodings[currentEncoding].testtext);

  myfont = f;
  setText( 0, myfont);
  setText( 1, testtext);
}

// This paints the Cell taking care to chage the font
void FontListItem::paintCell( QPainter *p, const QColorGroup &cg,
                                 int column, int width, int alignment )
{
  QFont font1;
  font1.setRawName (myfont);
  p->setFont(font1);
  QListViewItem::paintCell( p, cg, column, width, alignment );
}

//***************************************************************************

int main( int argc, char **argv )
{
    QApplication a( argc, argv );
    a.setDefaultCodec(
	QTextCodec::codecForName(knownEncodings[myencoding].codecname ));

    FontSelector t;
    t.setCaption( "X font selector v. " VERSION );
    a.setMainWidget( &t );
    t.show();
    t.resizeSelector();

    return a.exec();
}
