/*
Better X font selector using Qt library

Author: Jacek Pliszka <pliszka@fuw.edu.pl>, v0.1, 11 Oct 2000.
Patches: Enrique Perez-Terron <enrio@online.no>, v0.9, 30 Nov 2001.

Licence: GPL
 */
#ifndef XFONTSELECTOR_H
#define XFONTSELECTOR_H


class QListBox;
class QButtonGroup;

#include <qlist.h>
#include <qwidget.h>
#include <qsplitter.h>
#include <qcheckbox.h>
#include <qlistview.h>
#include <qtextcodec.h>

class FontInfo {
 public:
  QString font;
  QString family;
  QString name;
  QString weight;
  QString slant;
  QString width;
  QString size;
  int sizestart;
  int sizeend;
};

class FontInfoList: public QList<FontInfo>
{
 protected:
  int FontInfoList::compareItems ( FontInfoList::Item item1, FontInfoList::Item item2 );
};

class FontListItem : public QListViewItem
{
 public:
    FontListItem( QListView *parent, QString f );

    virtual void paintCell( QPainter *p, const QColorGroup &cg,
                           int column, int width, int alignment );

    QString font() { return myfont; }
protected:
    QString myfont;
};

class FontSelector: public QWidget
{
  Q_OBJECT
 public:
  FontSelector();
  void resizeSelector();
 public slots:
  void get_font_list(int);
  void showFonts();
  void printFonts();
  void help();
  void quit();
 protected:
  void   resizeEvent( QResizeEvent * );
 private:
  QCheckBox * scalablefonts;

  FontInfoList * fonts;
  QStringList * names;
  QStringList * sizes;
  QStringList * weights;
  QStringList * slants;
  QStringList * widths;
  
  QListBox * sizelist;
  QListBox * weightlist;
  QListBox * slantlist;
  QListBox * widthlist;
  QListBox * namelist;
  QListBox * fontslist;

  QWidget * fontdisplay;
  QListView * matchingfonts;

  QFrame * helpPopup;

  QSplitter * mainsplitter;
  QSplitter * verticalsplitter1;
  QSplitter * verticalsplitter2;
  //  QSplitter * verticalsplitter3;

  void write_out_matching_fonts(QString,QStringList*);
};


#endif
